@@ -36,18 +36,21 @@ handle_pinch_begin (MetaWaylandPointer *pointer,
                     const ClutterEvent *event)
 {
   MetaWaylandPointerClient *pointer_client;
+  MetaWaylandSeat *seat;
   struct wl_resource *resource;
-  uint32_t serial;
+  uint32_t serial, fingers;
 
   pointer_client = pointer->focus_client;
-  serial = wl_display_next_serial (pointer->seat->wl_display);
+  seat = meta_wayland_pointer_get_seat (pointer);
+  serial = wl_display_next_serial (seat->wl_display);
+  fingers = clutter_event_get_touchpad_gesture_finger_count (event);
 
   wl_resource_for_each (resource, &pointer_client->pinch_gesture_resources)
     {
       zwp_pointer_gesture_pinch_v1_send_begin (resource, serial,
                                                clutter_event_get_time (event),
                                                pointer->focus_surface->resource,
-                                               2);
+                                               fingers);
     }
 }
 
@@ -80,12 +83,14 @@ handle_pinch_end (MetaWaylandPointer *pointer,
                   const ClutterEvent *event)
 {
   MetaWaylandPointerClient *pointer_client;
+  MetaWaylandSeat *seat;
   struct wl_resource *resource;
   gboolean cancelled = FALSE;
   uint32_t serial;
 
   pointer_client = pointer->focus_client;
-  serial = wl_display_next_serial (pointer->seat->wl_display);
+  seat = meta_wayland_pointer_get_seat (pointer);
+  serial = wl_display_next_serial (seat->wl_display);
 
   if (event->touchpad_pinch.phase == CLUTTER_TOUCHPAD_GESTURE_PHASE_CANCEL)
     cancelled = TRUE;
