@@ -194,6 +194,7 @@ async_get_property_finish (xcb_connection_t          *xcb_conn,
 {
   xcb_get_property_reply_t *reply;
   xcb_generic_error_t *error;
+  int length;
 
   reply = xcb_get_property_reply (xcb_conn, cookie, &error);
   if (error)
@@ -209,8 +210,15 @@ async_get_property_finish (xcb_connection_t          *xcb_conn,
   results->prop = NULL;
 
   if (results->type != None)
-    results->prop = g_memdup (xcb_get_property_value (reply),
-                              xcb_get_property_value_length (reply));
+    {
+      length = xcb_get_property_value_length (reply);
+      /* Leave room for a trailing '\0' since xcb doesn't return null-terminated
+       * strings
+       */
+      results->prop = g_malloc (length + 1);
+      memcpy (results->prop, xcb_get_property_value (reply), length);
+      results->prop[length] = '\0';
+    }
 
   free (reply);
   return (results->prop != NULL);
@@ -296,6 +304,8 @@ motif_hints_from_results (GetPropertyResults *results,
 
   if (results->type == None || results->n_items <= 0)
     {
+      g_free (results->prop);
+      results->prop = NULL;
       meta_verbose ("Motif hints had unexpected type or n_items\n");
       return FALSE;
     }
@@ -306,10 +316,18 @@ motif_hints_from_results (GetPropertyResults *results,
    */
   *hints_p = calloc (1, sizeof (MotifWmHints));
   if (*hints_p == NULL)
-    return FALSE;
+    {
+      g_free (results->prop);
+      results->prop = NULL;
+      return FALSE;
+    }
 
   memcpy(*hints_p, results->prop, MIN (sizeof (MotifWmHints),
                                        results->n_items * sizeof (uint32_t)));
+
+  g_free (results->prop);
+  results->prop = NULL;
+
   return TRUE;
 }
 
@@ -341,6 +359,9 @@ latin1_string_from_results (GetPropertyResults *results,
 
   *str_p = g_strndup ((char *) results->prop, results->n_items);
 
+  g_free (results->prop);
+  results->prop = NULL;
+
   return TRUE;
 }
 
@@ -388,6 +409,9 @@ utf8_string_from_results (GetPropertyResults *results,
 
   *str_p = g_strndup ((char *) results->prop, results->n_items);
 
+  g_free (results->prop);
+  results->prop = NULL;
+
   return TRUE;
 }
 
@@ -507,7 +531,7 @@ window_from_results (GetPropertyResults *results,
   if (!validate_or_free_results (results, 32, XA_WINDOW, TRUE))
     return FALSE;
 
-  *window_p = *(Window*) results->prop;
+  *window_p = *(uint32_t *) results->prop;
   g_free (results->prop);
   results->prop = NULL;
 
@@ -523,7 +547,7 @@ counter_from_results (GetPropertyResults *results,
                                  TRUE))
     return FALSE;
 
-  *counter_p = *(XSyncCounter*) results->prop;
+  *counter_p = *(uint32_t *) results->prop;
   g_free (results->prop);
   results->prop = NULL;
 
@@ -764,7 +788,11 @@ size_hints_from_results (GetPropertyResults *results,
     return FALSE;
 
   if (results->n_items < OldNumPropSizeElements)
-    return FALSE;
+    {
+      g_free (results->prop);
+      results->prop = NULL;
+      return FALSE;
+    }
 
   raw = (xPropSizeHints*) results->prop;
 
